/* 
http://www.ksacvet777.moy.su
http://www.gamedev.ru/code/tip/VibrateCamera

  

          -
  .  -  .
      (  ),     .

     ,    ,
   .
    ""    .

  :
	   ,      ""
	     .


 ( time)     (   GetTime() * 100.0f ),
     ,   .

   "" .       .

 .


   " "
    

*/

#include <d3dx9.h>


void VibrateCamera( D3DXVECTOR3& eyeat, 
				   const D3DXVECTOR3& eyepos, 
				   const D3DXVECTOR3& eyeup, 
				   const float time, 
				   const float amplitude )
{
	D3DXVECTOR3 dir = eyeat - eyepos;
	D3DXVec3Normalize( &dir , &dir );
	D3DXVECTOR3 n_horiz;
	D3DXVec3Cross( &n_horiz, &dir, &eyeup );
	D3DXVECTOR3 n_vertic;
	D3DXVec3Cross(&n_vertic, &dir, &n_horiz);
	D3DXVec3Normalize( &n_vertic , &n_vertic );

	float noise_1 = amplitude * (sin( time  )         + sin( 3.78f - time * 1.254f )  + sin( 1.6f + time * 0.854f ));
	float noise_2 = amplitude * (sin( time * 1.78f  ) + sin( 2.78f - time * 1.254f )  + sin( 0.6f + time * 1.854f ));

	eyeat +=  n_horiz  * noise_1;
	eyeat +=  n_vertic * noise_2;

}



#include <mmsystem.h>

#pragma warning( disable : 4996  )
#pragma warning( disable : 4995 )

#include <strsafe.h>
 

#pragma comment(lib,"d3d9.lib")
#pragma comment(lib,"d3dx9.lib")
#pragma comment(lib,"winmm.lib")

#include <assert.h>
#include <assert.h>

#include <string>
#include <iostream>
#include <sstream>


LPDIRECT3D9             g_pD3D = NULL;  
LPDIRECT3DDEVICE9       g_pd3dDevice = NULL;  
LPDIRECT3DVERTEXBUFFER9 g_pVB = NULL;  
ID3DXFont* g_font = NULL;
float g_vibrateAmplitude =0.0f;


struct CUSTOMVERTEX
{
	FLOAT x, y, z; 
	DWORD color;  
};


#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE)



HRESULT InitD3D( HWND hWnd )
{

	if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
		return E_FAIL;


	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof( d3dpp ) );
	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;

	if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&d3dpp, &g_pd3dDevice ) ) )
	{
		return E_FAIL;
	}

	g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
 

	return S_OK;
}

 
HRESULT InitGeometry()
{
	CUSTOMVERTEX g_Vertices[] =
	{
		{ -1.0f,-1.0f, 0.0f, 0xffff0000, },
		{  1.0f,-1.0f, 0.0f, 0xff0000ff, },
		{  0.0f, 1.0f, 0.0f, 0xffffffff, },
	};

	if( FAILED( g_pd3dDevice->CreateVertexBuffer( 3 * sizeof( CUSTOMVERTEX ),
		0, D3DFVF_CUSTOMVERTEX,
		D3DPOOL_DEFAULT, &g_pVB, NULL ) ) )
	{
		return E_FAIL;
	}

	VOID* pVertices;
	if( FAILED( g_pVB->Lock( 0, sizeof( g_Vertices ), ( void** )&pVertices, 0 ) ) ) return E_FAIL;
	memcpy( pVertices, g_Vertices, sizeof( g_Vertices ) );
	g_pVB->Unlock();

	return S_OK;
}



VOID Cleanup()
{
	if( g_pVB != NULL )
		g_pVB->Release();

	if( g_font != NULL )
		g_font->Release();
	g_font = NULL;

	if( g_pd3dDevice != NULL )
		g_pd3dDevice->Release();

	if( g_pD3D != NULL )
		g_pD3D->Release();
}


float GetTime()
{
	static DWORD st_Start = 0;
	if(st_Start == 0)
	{
		st_Start = ::GetTickCount();
		return 0.0f;
	}

	DWORD temp = ::GetTickCount() - st_Start;
	return 0.001f * (float)temp;
}

void SetupMatrices()
{

	D3DXMATRIX matWorld;
 
	FLOAT fAngle = GetTime() * ( 2.0f * D3DX_PI );
	D3DXMatrixRotationY( &matWorld, fAngle );
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );


	D3DXVECTOR3 vEyePt( 20.0f, 25.0f, 30.0f );
	D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );

	VibrateCamera( vLookatPt, vEyePt, vUpVec, GetTime() * 100.0f, g_vibrateAmplitude );
	g_vibrateAmplitude *= 0.92f;
	if(g_vibrateAmplitude < 0.001f)   
		g_vibrateAmplitude=0.0f;

	D3DXMATRIXA16 matView;
	D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );
	g_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );


	D3DXMATRIXA16 matProj;
	D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI / 4, 1.0f, 1.0f, 100.0f );
	g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );
}


void RenderText()
{
	if(!g_font)
	{
		HRESULT hr = D3DXCreateFont( g_pd3dDevice, 16, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET,
			OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE,
			L"Arial", &g_font ) ;
		assert(g_font);
	}


}


HRESULT RenderCube()
{
	if (!g_pd3dDevice) return E_FAIL;
	HRESULT hr =0;

 
	struct CubeVertex
	{
		float x, y, z;
		D3DCOLOR c;
		FLOAT u, v;
	};

	// static Vert vert[];
	CubeVertex Vertexes[]  = 
	{
		{  -1.0f,    1.0f,   -1.0f,    0xffffffff,  0.0f, 0.0f,},
		{   1.0f,    1.0f,   -1.0f,    0xff00ffff,  1.0f, 0.0f,},
		{   1.0f,   -1.0f,   -1.0f,    0xffff00ff,  1.0f, 1.0f,},
		{  -1.0f,   -1.0f,   -1.0f,    0xffffffff,  0.0f, 1.0f,},

		{  -1.0f,    1.0f,    1.0f,    0xffffffff,  0.0f,   0.0f,},
		{  -1.0f,    1.0f,   -1.0f,    0xffffffff,  1.0f,   0.0f,},
		{  -1.0f,   -1.0f,   -1.0f,    0xffffffff,  1.0f,   1.0f,},
		{  -1.0f,   -1.0f,    1.0f,    0xffffffff,  0.0f,   1.0f,},

		{   1.0f,    1.0f,    1.0f,    0xffffffff,   0.0f,   0.0f,},
		{  -1.0f,    1.0f,    1.0f,    0xffffffff,   1.0f,   0.0f,},
		{  -1.0f,   -1.0f,    1.0f,    0xffffffff,   1.0f,   1.0f,},
		{   1.0f,   -1.0f,    1.0f,    0xffffffff,   0.0f,   1.0f,},

		{   1.0f,    1.0f,   -1.0f,    0xffffffff,   0.0f,   0.0f,},
		{   1.0f,    1.0f,    1.0f,    0xffff00ff,   1.0f,   0.0f,},
		{   1.0f,   -1.0f,    1.0f,    0xffffffff,   1.0f,   1.0f,},
		{   1.0f,   -1.0f,   -1.0f,    0xffff00ff,   0.0f,   1.0f,},

		{  -1.0f,    1.0f,    1.0f,    0xff00ffff,   0.0f,   0.0f,},
		{   1.0f,    1.0f,    1.0f,    0xffffffff,  1.0f,   0.0f,},
		{   1.0f,    1.0f,   -1.0f,    0xff00ffff,   1.0f,   1.0f,},
		{  -1.0f,    1.0f,   -1.0f,    0xffffffff,   0.0f,   1.0f,},

		{   1.0f,   -1.0f,   -1.0f,    0xffffffff,   0.0f,   0.0f,},
		{   1.0f,   -1.0f,    1.0f,    0xfffff00f,   1.0f,   0.0f,},
		{  -1.0f,   -1.0f,    1.0f,    0xffffffff,   1.0f,   1.0f,},
		{  -1.0f,   -1.0f,   -1.0f,    0xff00ffff,   0.0f,   1.0f,}
	};

	WORD Indexes[]   = {
		0,  1,  2,  2,  3,  0,
		4,  5,  6,  6,  7,  4,
		8,  9, 10, 10, 11,  8,
		12, 13, 14, 14, 15, 12,
		16, 17, 18, 18, 19, 16,
		20, 21, 22, 22, 23, 20
	};


	hr |= g_pd3dDevice->SetFVF( D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1  );
	hr |= g_pd3dDevice->DrawIndexedPrimitiveUP(	D3DPT_TRIANGLELIST, 0, 36, 12, 	(void*)Indexes,  D3DFMT_INDEX16, (void*)Vertexes, sizeof(CubeVertex) );
	assert(hr==0);
	return hr;
}

HRESULT RenderScene()
{
	if(!g_pd3dDevice) return E_FAIL;

	D3DXMATRIX ms, mt, mw;
 
	D3DXMatrixScaling(&ms, 1.0f, 2.0f, 1.0f);
	D3DXMatrixTranslation(&mt, 10.0f , 2.0f , 10.0f);
	mw = ms * mt;
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &mw );
	RenderCube();
 
	D3DXMatrixScaling(&ms, 1.0f, 2.0f, 1.0f);
	D3DXMatrixTranslation(&mt, 10.0f , 2.0f , 10.0f);
	mw = ms * mt;
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &mw );
	RenderCube();

	D3DXMatrixScaling(&ms, 1.0f, 2.0f, 1.0f);
	D3DXMatrixTranslation(&mt, 10.0f , 2.0f , -10.0f);
	mw = ms * mt;
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &mw );
	RenderCube();

	D3DXMatrixScaling(&ms, 1.0f, 2.0f, 1.0f);
	D3DXMatrixTranslation(&mt, -10.0f , 2.0f , 10.0f);
	mw = ms * mt;
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &mw );
	RenderCube();

	D3DXMatrixScaling(&ms, 1.0f, 2.0f, 1.0f);
	D3DXMatrixTranslation(&mt, -10.0f , 2.0f , -10.0f);
	mw = ms * mt;
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &mw );
	RenderCube();

	return S_OK;
}

void KeyPress( int vkey )
{
	if(vkey == ' ')
	{
		g_vibrateAmplitude = 0.9f; 
	}
}


void Render()
{
 
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB( 0, 0, 0 ), 1.0f, 0 );

 
	g_pd3dDevice->BeginScene();
 
	SetupMatrices();
 
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
	g_pd3dDevice->SetStreamSource( 0, g_pVB, 0, sizeof( CUSTOMVERTEX ) );
	g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 1 );

	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
	RenderScene();

	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
	RenderText();
 
	g_pd3dDevice->EndScene();


 
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}


LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_DESTROY:
		Cleanup();
		PostQuitMessage( 0 );
		return 0;


		case WM_CHAR:
			{
				KeyPress( (int)wParam );

			}
			break;
	}

	return DefWindowProc( hWnd, msg, wParam, lParam );
}



int main()
{
	WNDCLASSEXW wc =
	{
		sizeof( WNDCLASSEX ), CS_CLASSDC, MsgProc, 0L, 0L,
		GetModuleHandle( NULL ), NULL, 
		::LoadCursor(0, IDC_ARROW),
		NULL, NULL,
		L"D3D9", NULL
	};
	RegisterClassEx( &wc );

	HWND hWnd = CreateWindowW( L"D3D9", L"D3D9 renderer",
		WS_OVERLAPPEDWINDOW, 1, 1, 1024, 800,
		NULL, NULL, wc.hInstance, NULL );

	if( SUCCEEDED( InitD3D( hWnd ) ) )
	{
		if( SUCCEEDED( InitGeometry() ) )
		{
			ShowWindow( hWnd, SW_SHOWDEFAULT );
			UpdateWindow( hWnd );

			MSG msg;
			ZeroMemory( &msg, sizeof( msg ) );
			while( msg.message != WM_QUIT )
			{
				if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
				{
					TranslateMessage( &msg );
					DispatchMessage( &msg );
				}
				else
					Render();
			}
		}
	}

	UnregisterClassW( L"D3D9", wc.hInstance );
	return 0;
}

